/*-----------------------------------------------------------------------

File  : can_infstate.h

Author: Stephan Schulz

Contents
 
  Data structures for representing the state of an inference process. 

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Feb 26 23:25:00 MET 1999
    New

-----------------------------------------------------------------------*/

#ifndef CAN_INFSTATE

#define CAN_INFSTATE

#include <ccl_inferencedoc.h>
#include <ccl_clausesets.h>
#include <can_clausestore.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/


/* Represent the state of an inference process. This is a kind of
   strange date structure, because it represents the _result_ of a
   series of inference steps, not the steps themselves. */

typedef struct infstatecell
{
   TB_p        terms;
   PDArray_p   clause_store;
   PDArray_p   term_clause_assoc;
   NumTree_p   watch_clauses;

   TB_p        axiom_terms;        /* We want to be able to store */
   ClauseSet_p axioms;             /* the axioms in original form */
   bool        generate_pcl;       /* Obvious ;-) */
   bool        generate_pcl_all;   /* Otherwise only use watched
				      clauses */
   bool        print_proc_pcl;     /* Print PCL for processing steps */
   bool        print_clause_evals; /* Print clauses on parsing a CE
		  		      statement */
   bool        print_clause_mods;  /* Print clauses whenever they are
		  		      changed or created by an
				      inference */
   bool        print_clause_stats; /* Print clause statistics with
				      clause - only for clause output
				      during parsing */ 
   int         update_inf_inc;     /* Will be added to values below on
                                      each appropriate inference - set
                                      to 0 to ignore !*/
   long        created_count;      /* Counts CC steps */
   long        processed_count;    /* Counts CP steps */   
   long        ext_id_source;      /* This is incremented whenever a
				      clause gets a new external
				      identifier for PCL generation */
   NumTree_p   ac_axioms;          /* Stores (pcl-ident, clause,
				      original ident) triples for
				      original AC axioms */
}InfStateCell, *InfState_p;



#define PCL_QUOTE NULL
#define PCL_ER    "er"
#define PCL_PM    "pm"
#define PCL_EF    "ef"
#define PCL_URW   "urw"
#define PCL_RW    "rw"
#define PCL_SR    "sr"
#define PCL_ACRES "ar"
#define PCL_CN    "cn"


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

#define InfStateCellAlloc()\
        (InfStateCell*)SizeMalloc(sizeof(InfStateCell))
#define InfStateCellFree(junk)\
        SizeFree(junk, sizeof(InfStateCell))

InfState_p InfStateAlloc(void);
void       InfStateFree(InfState_p state);

void       InfStateClausesDelProp(InfState_p state, ClauseProperties
				  props);
void       InfStateClausesSetProp(InfState_p state, ClauseProperties
				  props);

void       InfStateAddTermAssoc(InfState_p state, CompClause_p
				clause);
void       InfStateRemoveTermAssoc(InfState_p state, CompClause_p
				   clause);

void       InfStateInferenceParse(Scanner_p in, FILE* out, InfState_p state);
long       InfStateInfListParse(Scanner_p in, FILE* out, InfState_p state);

void InfStatePrintClauses(FILE* out, InfState_p state,
			  ClauseProperties filter, bool status, bool
			  deps, bool stats);

long InfStateStoreClauseInfo(NumTree_p *store, InfState_p state);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





