/*-----------------------------------------------------------------------

File  : clb_123trees.c

Author: Stephan Schulz

Contents
 
  Experimental new data structure for managing pointer sets.

Changes

<1> Tue Jan 26 19:42:24 MET 1999
    New

-----------------------------------------------------------------------*/

#include "clb_123trees.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------
//
// Function: X123TreeCellAllocEmpty()
//
//   Allocate an empty, initialized 123-tree cell
//
// Global Variables: -
//
// Side Effects    : Memory operations
//
/----------------------------------------------------------------------*/

X123Tree_p X123TreeCellAllocEmpty()
{
   X123Tree_p handle = X123TreeCellAlloc();
   int i;

   X123CellDelFull(handle);
   
   for(i=0; i<X123ENTRY_NO; i++)
   {
      handle->entry[i] = NULL;
      X123EntryDelSon(handle,i);
   }   
   return handle;
}


/*-----------------------------------------------------------------------
//
// Function: X123TreeFree()
//
//   Free a 123 tree (do not free objects pointed to)
//
// Global Variables: -
//
// Side Effects    : Memory management
//
/----------------------------------------------------------------------*/

void X123TreeFree(X123Tree_p junk)
{
   if(junk)
   {
      int i;
      for(i=0; i<X123ENTRY_NO; i++)
      {
	 if(junk->entry[i] && X123EntryIsSon(junk,i))
	 {
	    X123TreeFree(junk->entry[i]);
	 }
      }
      X123TreeCellFree(junk);
   }
}


/*-----------------------------------------------------------------------
//
// Function: X123TreeInsert() 
//
//   Insert a new element into a 123tree. Return true if the key
//   already exists in the tree.
//
// Global Variables: -
//
// Side Effects    : Memory operations, changes tree.
//
/----------------------------------------------------------------------*/

bool X123TreeInsert(X123Tree_p *root, X123Tree_p new)
{
   if(!*root)
   {
      *root = X123TreeCellAllocEmpty();
      (*root)->entry[0] = new;
      assert(!X123EntryIsSon(*root,0));
   }
   else
   {
      if(!X123CellIsFull(*root))
      {
	 array_insert(*root, new);
      }
      else
      {
	 grow_insert(*root, new);
      }
   }
}


bool       X123TreeStore(X123Tree_p *root, void* key);
X123Tree_p X123TreeFind(X123Tree_p root, void* key);
X123Tree_p X123TreeExtractEntry(X123Tree_p *root, void* key);
void*      X123TreeExtractKey(X123Tree_p *root, void* key);
void*      X123TreeExtractRootKey(X123Tree_p *root);
bool       X123TreeDeleteEntry(X123Tree_p *root, void* key);
void       X123TreeMerge(X123Tree_p *root, X123Tree_p add);
long       X123TreeNodes(X123Tree_p root);

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


