/*-----------------------------------------------------------------------

File  : ccl_inferencedoc.h

Author: Stephan Schulz

Contents
 
  Functions and constants for reporting on the proof process.

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Tue Jan  5 15:27:37 MET 1999
    Partially new, partially lifted from ccl_clauses.[ch]

-----------------------------------------------------------------------*/

#ifndef CCL_INFERENCEDOC

#define CCL_INFERENCEDOC

#include <ccl_clauses.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

/* Symbolic constants for the relevant annotations of the output
   format. These should better be valid tokens of type Identifier,
   otherwise some parsing code might break. Except for this
   constraint, change them around as much as you like (but don't
   expect to be compatible with the official release). */ 

#define CC_AXIOM            "AXIOM"
#define CC_PARAMOD          "PM"
#define CC_EFACTOR          "EF"
#define CC_OFACTOR          "OF"
#define CC_ERESOLUTION      "ER"

#define CM_SIMPLIFY_REFLECT "SR"
#define CM_TERM_RENAME      "TR"
#define CM_MINIMIZE         "MR"
#define CM_ACRESOLVE        "ACRES"

#define CP_PROCESS          "SELECT"
#define CP_PROOF            "PROOF"
#define CP_FINAL            "FINAL"
#define CP_EXISTS           "EXISTS"

#define CE_EVAL             "CE"

#define CS_DELETE           "CD"
#define CS_RENAME           "CR"
#define CS_SUBSUME          "SS"

#define AC_REGISTER         "AC"

#define RW_STANDARD         "RW"


typedef enum 
{
   inf_noinf,
   inf_initial,
   inf_quote,
   inf_paramod,
   inf_efactor,
   inf_eres,
   inf_simplify_reflect,
   inf_ac_resolution,
   inf_minimize,
   inf_rewrite,
   inf_mult_rewrite,
   inf_quote_proc,
   inf_quote_eval,
   inf_quote_final,
   inf_quote_exists
}InfType;
   
   


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

void    ClauseDocumentCreation(FILE* out, long level, Clause_p
			       clause, char* operation);
#define ClauseDefaultDocumentCreation(clause, operation)\
        ClauseDocumentCreation(GlobalOut, OutputLevel, (clause),\
		       (operation))
   
void    ClauseRename(FILE* out, long level, Clause_p clause, long
			new_ident); 

void    ClauseDocumentStatusChange(FILE* out, long level, Clause_p
				   clause, char* operation);
#define ClauseDefaultDocumentStatusChange(clause, operation)\
        ClauseDocumentStatusChange(GlobalOut, OutputLevel, (clause),\
				   (operation))
void    ClauseDocumentModification(FILE* out, long level, Clause_p
				   clause, char* operation, Clause_p
				   partner);
#define ClauseDefaultDocumentModification(clause, operation, partner)\
        ClauseDocumentModification(GlobalOut, OutputLevel, (clause),\
				   (operation), (partner))

void    ClauseDocumentProcessing(FILE* out, long level, Clause_p
				 clause, char* operation);
#define ClauseDefaultDocumentProcessing(clause, operation)\
        ClauseDocumentProcessing(GlobalOut, OutputLevel, (clause),\
				 (operation))

void    ClauseDocumentEvaluation(FILE* out, long level, Clause_p
				 clause); 
#define ClauseDefaultDocumentEvaluation(clause)\
        ClauseDocumentEvaluation(GlobalOut, OutputLevel, (clause))

void    ClauseDocumentSubsumption(FILE* out, long level, Clause_p
				  subsumer, Clause_p subsumed); 
#define ClauseDefaultDocumentSubsumption(subsumer, subsumed)\
        ClauseDocumentSubsumption(GlobalOut, OutputLevel, \
				  (subsumer),(subsumed))

void ClauseDocumentAC(FILE* out, long level, Clause_p clause);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





